import os


def get_log_stream_html(title = "Log streamer", ws_url = None):
    path = os.path.abspath(os.path.join(
        __file__,
        os.pardir,
        "LogStreaming.html"
    ))
    with open(path, encoding="utf-8") as file:
        html = file.read()

    html = html.replace("{{ENV_TITLE}}", title)

    return html


class Hour:
    """Using for validating hour and preparing previous or next hour of hour
    """
    hour: int
    def __init__(self, hour):
        self.validate_hour(hour)
        self.hour = hour
    
    def __repr__(self) -> str:
        return f"<UtilHour: {self.hour}>"
    
    def __str__(self) -> str:
        if self.hour <= 9:
            return f"0{self.hour}"
        return f"{self.hour}"

    @property
    def next_hour(self) -> "Hour":
        hour = self.hour + 1
        if hour >= 24:
            return self.__class__(0)
        return self.__class__(hour)

    @property
    def previous_hour(self) -> "Hour":
        hour = self.hour - 1
        if hour <= -1:
            return self.__class__(23)
        return self.__class__(hour)
    
    @staticmethod
    def validate_hour(hour: int) -> None:
        if hour >= 24 or hour < 0:
            raise ValueError("Hour can be more then 0 and less then 24.")


# Datetime formats ---- >
DUMP_LOGFILE_DATE_FORMAT = "%Hh %d-%m-%Y"
FILE_DATE_FORMAT = "%H.%M"
FOLDER_DATE_FORMAT = "%Y-%m-%d"
LOGSTRING_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"
REQUEST_DATESTRING_FORMAT = "%H:%M %d/%m/%Y"

# Logger Formatter ---- >
DEFAULT_FORMATTER = "[{datetime}] {levelname} in {filename} line {line}: {message}"

# RegExp for get log from logline ---- >
LOGSTRING_DATE_REGEXP = r'\[(\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2})\]'

# Max attempts for read logline date ---- >
LOGFILE_ITERATE_MAX_ATTEMPTS = 30

# Default settings ---- >
DEFAULT_ENCODING="utf-8"
DEFAULT_LOGGING_FILENAME="logging.log"
DEFAULT_LOGGING_LEVEL="INFO"
DEFAULT_PATH="logs"



from abc import ABC

from .abstract import AbstractLoggerConfig
from .utils import ConfigLogger

class UvicornLogger(AbstractLoggerConfig, ABC):
    def _use_uvicorn(self) -> None:
        self.loggers.append(ConfigLogger(
            name="uvicorn.access",
            level=self.settings.logger_level,
            handlers=self.handlers,
            propagate=False,
        ))

        self.loggers.append(ConfigLogger(
            name="uvicorn.error",
            level=self.settings.logger_level,
            handlers=self.handlers,
            propagate=False,
        ))


from datetime import datetime, timedelta
from fastapi import WebSocket

from MeowthLogger.constants import REQUEST_DATESTRING_FORMAT
from MeowthLogger.utilities.dates import set_null_minutes

from ..log_streaming.stream_manager import Stream

class StreamManager(Stream):
    def __init__(self, loop):
        self.active_connections: list[WebSocket] = []
        self.loop = loop
    
    @property
    def _prev_logs(self):
        prev_date = datetime.now()
        prev_date = set_null_minutes(prev_date) - timedelta(hours=1)
        prev_date = datetime.strftime(prev_date, REQUEST_DATESTRING_FORMAT)
        file = self.logger.stream_logs(date_from=prev_date)

        logs_lines = []
        while True:
            line = file.readline().decode('utf-8')
            if not line:
                break
            logs_lines.append(line)
        return "ROOTMSG" + "".join([
            line
            for line in logs_lines[-1000:]
        ])

    async def connect(self, websocket: WebSocket):
        await websocket.accept()
        await websocket.send_text(self._prev_logs)
        self.active_connections.append(websocket)

    def disconnect(self, websocket: WebSocket):
        self.active_connections.remove(websocket)

    def write(self, message):
        self.loop.create_task(self.broadcast(message))

    async def broadcast(self, message: str):
        for connection in self.active_connections:
            await connection.send_text(message)


import re

from datetime import datetime, timedelta

from MeowthLogger.errors import (
    NotDateLogString,
    NotValidLogsDateFormat,
)
from MeowthLogger.constants import (
    LOGSTRING_DATE_FORMAT,
    LOGSTRING_DATE_REGEXP,
    REQUEST_DATESTRING_FORMAT,
)

def convert_request_datestring(request_date_string: str) -> datetime:
    """Converting reques string date to datetime object
    """
    try:
        d = datetime.strptime(request_date_string, REQUEST_DATESTRING_FORMAT)
        return d
    except:
        raise NotValidLogsDateFormat

def set_null_minutes(date: datetime) -> datetime:
    """Preparing date for get datetime hour object
    """
    return date - timedelta(
        minutes=date.minute,
        seconds=date.second,
        microseconds=date.microsecond
    )

def date_from_logstring(log_string: str) -> datetime:
    """Preparing date in logstring date format to datetime object
    """
    date_pattern = re.compile(LOGSTRING_DATE_REGEXP)
    match = date_pattern.search(log_string)

    if not match:
        raise NotDateLogString

    return datetime.strptime(
        match.group(1),
        LOGSTRING_DATE_FORMAT
    )


import asyncio
from MeowthLogger import Logger

from fastapi import FastAPI

from MeowthLogger.utilities.fastapi.log_stream import StreamManager
from MeowthLogger.utilities.fastapi.views import get_log_stream_views_router


loop = asyncio.new_event_loop()
managa = StreamManager(loop)
api = FastAPI()


@api.get("/")
async def home():
    return {"ok": True}

logger = Logger(use_files=True, use_uvicorn=True, stream=managa)
logger.info("TEST")


api.include_router(
    get_log_stream_views_router(logger)
)

from uvicorn import Config, Server

config = Config(
    app=api,
    host="0.0.0.0",
    log_config=None
)

server = Server(config)

async def main():
    await server.serve()

if __name__ == "__main__":
    # loop.create_task(server.serve())
    # loop.run_forever()
    asyncio.run(main())


import logging

from colorama import Back, Fore, Style

from .formatter import AbstractFormatter

class ColorisedFormatter(AbstractFormatter):
    def prepare_log_string(
            self,
            datetime,
            levelname,
            filename,
            line,
            message,
        ):

        datetime = f"{Style.DIM}{datetime}{Style.NORMAL}"
        filename = f"{Fore.LIGHTYELLOW_EX}{filename}{Fore.RESET}"
        line = f"{Fore.LIGHTYELLOW_EX}{line}{Fore.RESET}"

        return super().prepare_log_string(
            datetime,
            levelname,
            filename,
            line,
            message,
        )
    
    def prepare_levelname(self, levelname):
        match levelname:
            case logging.INFO:
                return f"{Fore.GREEN}INFO{Fore.RESET}"
            case logging.ERROR:
                return f"{Fore.RED}ERROR{Fore.RESET}"
            case logging.WARN:
                return f"{Fore.YELLOW}WARNING{Fore.RESET}"
            case logging.DEBUG:
                return f"{Fore.LIGHTBLACK_EX}DEBUG{Fore.RESET}"
            case logging.CRITICAL:
                return f"{Back.RED} CRITICAL {Back.RESET}"
            case _:
                return f"LEVEL :{levelname}"


from abc import ABC

from .abstract import AbstractLoggerConfig
from .utils import (
    ConfigFormatter,
    ConfigHandler,
    ConfigLogger,
)

class ConfigFileHandler(ConfigHandler):
    def __init__(
        self,
        when: str,
        encoding: str,
        path: str,
        max_log_alive_time: int,
        *args, **kwargs
    ):
        self.when = when
        self.encoding = encoding
        self.path = path
        self.max_log_alive_time = max_log_alive_time
        super().__init__(*args, **kwargs)
    
    def json(self) -> dict:
        json = super().json()
        json.update({
            "when": self.when,
            "encoding": self.encoding,
            "path": self.path,
            "max_log_alive_time": self.max_log_alive_time,
        })
        return json
    
class FileLogger(AbstractLoggerConfig, ABC):
    def _use_files(self) -> None:
        default_formatter = ConfigFormatter(
            name="default", 
            class_name="MeowthLogger.formatters.DefaultFormatter",
        )
        self.formatters.append(default_formatter)

        file_handler = ConfigFileHandler(
            name="file",
            class_name="MeowthLogger.handlers.FileHandler",
            level=self.settings.logger_level,
            formatter=default_formatter,
            when="midnight",
            encoding=self.settings.encoding,
            path=self.settings.path,
            max_log_alive_time=self.settings.log_alive_seconds,
        )
        self.handlers.append(file_handler)

        file_logger = ConfigLogger(
            name="file",
            level=self.settings.logger_level,
            propagate=False,
            handlers=[file_handler],
        )
        self.loggers.append(file_logger)



import os
import shutil

from datetime import datetime, timedelta
from logging.handlers import TimedRotatingFileHandler

from MeowthLogger.errors import NotValidFile
from MeowthLogger.constants import (
    DEFAULT_LOGGING_FILENAME,
    FOLDER_DATE_FORMAT,
    LOGFILE_ITERATE_MAX_ATTEMPTS,
)
from MeowthLogger.utilities.dates import (
    date_from_logstring,
    set_null_minutes,
)
from MeowthLogger.utilities.custom_hour import Hour
from MeowthLogger.logger.parser.files import DirectoriesTree


class LogFile:
    def __init__(self, path):
        self.path = path
    
    def get_last_date(self):
        with open(self.path, "rb") as file:
            lines = file.readlines()
            try:
                date = self.find_date(lines)
            except:
                return datetime.now()
        return date
    
    def find_date(self, lines: list[str]) -> datetime:
        attempts = 0
        for line in lines[::-1]:
            if attempts >= LOGFILE_ITERATE_MAX_ATTEMPTS:
                raise NotValidFile(self.path)
            try:
                return date_from_logstring(
                    line.decode("utf-8")
                )
            except:
                continue
        
        raise NotValidFile(self.path)

class FileHandler(TimedRotatingFileHandler):
    def __init__(self, path, max_log_alive_time, *args, **kwargs):
        self.max_log_alive_time = max_log_alive_time
        self.path = path
        self.generate_root_path()
        super().__init__(
            filename=os.path.join(self.path, DEFAULT_LOGGING_FILENAME), 
            *args, **kwargs
        )
        self.start_rollover()



    # Rollovering block ---- >
    def start_rollover(self) -> None:
        now = set_null_minutes(datetime.now())
        last_date = set_null_minutes(self.last_logstream_date)
        if last_date < now:
            self.doRollover()

    def doRollover(self) -> None:
        self.close_stream()
        if self.max_log_alive_time != None:
            self.clear_files()

        date_from = set_null_minutes(self.last_logstream_date) \
            + timedelta(hours=1)

        file_path = self.generate_file_path(date_from)
        self.rotate(self.baseFilename, self.rotation_filename(file_path))
        self.open_stream()
        self.rolloverAt = self.computeRollover()

    def computeRollover(self, *args) -> datetime:
        now = datetime.now()
        rollover_at = now - timedelta(minutes=now.minute, seconds=now.second, microseconds=now.microsecond) + timedelta(hours=1)
        self.rolloverAt = rollover_at
        return rollover_at

    def shouldRollover(self, *args) -> bool:
        if not self.is_exists_base_file():
            self.rolloverAt = self.computeRollover()
            return False
        elif datetime.now() >= self.rolloverAt:
            return True
        return False
    # Rollovering block ---- <



    # Clearing block ---- >
    def clear_files(self):
        delta = timedelta(seconds=self.max_log_alive_time)
        dt_del = datetime.now() - delta

        tree = DirectoriesTree(self.path)

        filtered = list(filter(lambda dir: dir.date <= dt_del, tree.dirs))


        if filtered:
            while len(filtered) > 1:
                path = filtered.pop(0)
                shutil.rmtree(path.path_join)
        
            files = filtered[0].files
            files = list(filter(lambda file: file.date < dt_del, files))

            while len(files):
                file = files.pop()
                os.remove(file.path_join)
    # Clearing block ---- <



    # Streaming block ---- >>
    def close_stream(self) -> None:
        try:
            self.stream.close()
        except:
            pass
        self.stream = None
    
    def open_stream(self) -> None:
        self.stream = self._open()
    # Streaming block ---- <<



    # Files pathes block ---- >>

    # Base file ---- >
    @property
    def base_file_path(self) -> str:
        return os.path.join(
            self.path, DEFAULT_LOGGING_FILENAME
        )

    def is_exists_base_file(self) -> bool:
        return os.path.exists(self.base_file_path)
    # Base file ---- <


    # Path generators ---- >
    def generate_root_path(self) -> None:
        if not os.path.exists(self.path):
            os.makedirs(self.path)

    def generate_folder_path(self, date: datetime) -> str:
        folder_path = os.path.join(
            self.path, self.prepare_foldername(date)
        )
        
        if not os.path.exists(folder_path):
            os.mkdir(folder_path)

        return folder_path

    def generate_file_path(self, date: datetime) -> str:
        folder_path = self.generate_folder_path(date)

        file_path = os.path.join(
            folder_path, 
            self.prepare_log_filename(Hour(date.hour))
        )

        if os.path.exists(file_path):
            os.remove(file_path)

        return file_path
    # Path generators ---- <


    # Path prepearers ---->
    @staticmethod
    def prepare_foldername(date: datetime) -> str:
        return date.strftime(FOLDER_DATE_FORMAT)
    
    @staticmethod
    def prepare_log_filename(hour: Hour) -> str:
        return f"{hour.previous_hour}.00-{hour}.00.log"
    # Path prepearers ----<

    # Files pathes block ---- <<



    # Utils ---- >>
    @property
    def last_logstream_date(self) -> datetime:
        log_file = LogFile(self.base_file_path)

        try:
            date = log_file.get_last_date()
        except:
            return datetime.now()
        
        return date
    # Utils ---- <<


from datetime import datetime, timedelta
import io

from MeowthLogger.constants import (
    DEFAULT_LOGGING_FILENAME,
    DUMP_LOGFILE_DATE_FORMAT,
)
from MeowthLogger.errors import LoggerNotUsingFileSystem
from MeowthLogger.logger.settings import LoggerSettings
from MeowthLogger.utilities.dates import (
    convert_request_datestring,
    set_null_minutes,
)

from .files import (
    DirectoriesTree,
    RootLogFile,
)

class LogParser:
    """### Parser for load logs from logs directory and return streaming
    #### Usage:
    ```
    parser = LogParser(settings)
    date_from = "13:00 01/01/1999"
    stream: io.BytesIO = parser.stream_logs(date_from)

    date_to = "13:00 01/01/2099"
    stream: io.BytesIO = parser.stream_logs(date_from, date_to)
    ```

    #### Stream name was looking as "(13h 01-01-1999) (13h 01-01-2099).log"
    U can add custom headder for filename
    ```
    stream = parser.stream_logs(date_from, stream_name_header="LOGGER!")
    ```
    Filename should be looking as "LOGGER!(13h 01-01-1999) (13h 01-01-2099).log"

    Or set custom name
    ```
    stream = parser.stream_logs(date_from)
    stream.name = "LOGGER.log"
    ```
    """
    settings: LoggerSettings
    
    @classmethod
    def prepare_dates(cls, date_from: str, date_to: str) -> tuple[datetime, datetime]:
        """Preparing dates before load logs files
        """
        date_from = set_null_minutes(
            convert_request_datestring(date_from)
        )

        date_to = set_null_minutes(
            convert_request_datestring(date_to) if date_to
            else datetime.now()
        ) + timedelta(hours=1)

        if date_from >= date_to:
            raise ValueError("Not valid dates")

        return date_from, date_to
    
    @staticmethod
    def prepare_filename(
        date_from: datetime,
        date_to: datetime,
        header: str = None
    ) -> str:
        """Preparing filename for dumping file
        """

        # func for prepare dates to string
        def prepare_date(date: datetime) -> str:
            return date.strftime(DUMP_LOGFILE_DATE_FORMAT)
        
        return "" + \
            (header if header else "") + \
            f"({prepare_date(date_from)})" + \
            " " + \
            f"({prepare_date(date_to)})" + \
            ".log"

    def stream_logs(
            self,
            date_from: datetime,
            date_to: datetime = None,
            stream_name_header:str = None
    ) -> io.BytesIO:
        self.__validate_settings()

        date_from, date_to = self.prepare_dates(date_from, date_to)

        # Initialise bytes for writing in
        dump_file = io.BytesIO()

        dir_tree = DirectoriesTree(self.settings.path)
        if dir_tree.dirs:
            dir_tree.sort_files(date_from, date_to)
            
        for dir in dir_tree.dirs:
            dump_file.write(dir.read())

        if date_to > datetime.now():
            root_logfile = RootLogFile(DEFAULT_LOGGING_FILENAME, self.settings.path)
            dump_file.write(root_logfile.read())

        # change filename
        dump_file.name = self.prepare_filename(date_from, date_to, stream_name_header)
        # saving file
        dump_file.seek(0)

        return dump_file

    def __validate_settings(self) -> None:
        if not self.settings.use_files:
            raise LoggerNotUsingFileSystem


from abc import ABC

from MeowthLogger.utilities.abstractions import Dictable

from .utils import (
    ConfigFormatter,
    ConfigHandler,
    ConfigLogger,
    ConfigRoot,
)
from ..settings import LoggerSettings

class AbstractLoggerConfig(Dictable, ABC):
    version: int
    settings: LoggerSettings

    formatters: list[ConfigFormatter]
    handlers: list[ConfigHandler]
    loggers: list[ConfigLogger]

    root: ConfigRoot

    def __init__(
        self,
        settings: LoggerSettings,
        version: int,
    ):
        self.settings = settings
        self.version = version

        self.handlers = []
        self.loggers = []
        self.formatters = []

    def json(self) -> dict:
        return {
            'version': self.version,
            'formatters': {
                formatter.name: formatter.json()
                for formatter
                in self.formatters
            },
            'handlers': {
                handler.name: handler.json()
                for handler
                in self.handlers
            },
            'loggers': {
                logger.name: logger.json()
                for logger
                in self.loggers
            },
            'root': ConfigRoot(
                self.settings.logger_level,
                self.handlers
            ).json(),
        }


from abc import ABC

from .abstract import AbstractLoggerConfig
from .console import ConfigConsoleHandler
from .utils import (
    ConfigFormatter,
    ConfigLogger,
)

class StreamLogger(AbstractLoggerConfig, ABC):
    def _use_stream(self) -> None:
        formatter = ConfigFormatter(
            name="ws_stream", 
            class_name="MeowthLogger.formatters.DefaultFormatter",
        )
        self.formatters.append(formatter)

        handler = ConfigConsoleHandler(
            name="ws_stream",
            class_name="logging.StreamHandler",
            level=self.settings.logger_level,
            formatter=formatter,
            stream=self.settings.stream,
        )
        self.handlers.append(handler)

        logger = ConfigLogger(
            name="console",
            level=self.settings.logger_level,
            propagate=False,
            handlers=[handler],
        )
        self.loggers.append(logger)


import os

from abc import ABC
from datetime import datetime

class Dictable(ABC):
    """Abstraction for jsonify objects
    """
    def json(self) -> dict:
        raise NotImplementedError

class DateNameFile(ABC):
    """Absctraction for read files path and compare by dates
    """
    path: str
    name: str
    date: datetime

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} '{self.name}'>"

    def __lt__(self, other) -> bool:
        if isinstance(other, self.__class__):
            return self.date < other.date
        raise ValueError(f"Other not {self.__class__.__name__} class")

    @property
    def path_join(self) -> str:
        return os.path.join(self.path, self.name)


from MeowthLogger.utilities.abstractions import Dictable

class ConfigFormatter(Dictable):
    def __init__(self, 
        name: str,
        class_name: str
    ):
        self.name = name
        self.class_name = class_name

    def json(self) -> dict:
        return {
            "()": self.class_name,
        }

class ConfigHandler(Dictable):
    def __init__(self, 
        name: str, 
        class_name: str,
        formatter: ConfigFormatter,
        level: str
    ):
        self.name = name
        self.class_name = class_name
        self.formatter = formatter
        self.level = level

    def json(self) -> dict:
        return {
            "class": self.class_name,
            "formatter": self.formatter.name,
            "level": self.level,
        }

class ConfigLogger(Dictable):
    def __init__(
        self,
        name: str,
        level: str,
        handlers: list[ConfigHandler],
        propagate: bool
    ):
        self.name = name
        self.level = level
        self.handlers = handlers
        self.propagate = propagate
    
    def json(self) -> dict:
        return {
            "level": self.level,
            "propagate": self.propagate,
            "handlers": [
                handler.name
                for handler 
                in self.handlers
            ],
        }
class ConfigRoot(Dictable):
    def __init__(
        self,
        level: str,
        handlers: list[ConfigHandler]
    ):
        self.level = level
        self.handlers = handlers
    
    def json(self) -> dict:
        return {
            "level": self.level,
            "handlers": [
                handler.name
                for handler
                in self.handlers
            ],
        }


class NotDateLogString(Exception):
    def __init__(self) -> None:
        super().__init__("It's not date log string")

class NotValidFile(ValueError):
    def __init__(self, path) -> None:
        super().__init__(f"File {path} is not valid log file")

class NotValidLogsDateFormat(ValueError):
    def __init__(self) -> None:
        super().__init__("Not valid datetime string format, format is hh:mm DD/MM/YYYY")

class LoggerNotUsingFileSystem(Exception):
    def __init__(self) -> None:
        super().__init__("Logger not using file system")


from typing import Optional
from ..utilities.log_streaming.stream_manager import Stream

class LoggerSettings:
    """### Settings of root logger
    #### Parameters
    - logger_level: level of logger. Example: "INFO" or "DEBUG"
    - use_files: bool argument for use with files mode or without
    - encoding: file encoding mode. Example: "utf-8"
    - path: logging files path. Example: "logs/logfiles"
    - use_uvicorn: bool argument for use with uvicorn
    """
    def __init__(
        self,
        logger_level: str,
        use_files: bool,
        log_alive_seconds: Optional[int],
        encoding: str,
        path: str,
        use_uvicorn: bool,
        stream: Optional[Stream]
    ):
        self.logger_level = logger_level
        self.use_files = use_files
        self.log_alive_seconds = log_alive_seconds
        self.encoding = encoding
        self.path = path
        self.use_uvicorn = use_uvicorn
        self.stream = stream


from abc import ABC

from .abstract import AbstractLoggerConfig
from .utils import (
    ConfigFormatter,
    ConfigHandler,
    ConfigLogger,
)

class ConfigConsoleHandler(ConfigHandler):
    def __init__(self, 
        stream: str, 
        *args, **kwargs
    ):
        self.stream = stream
        super().__init__(*args, **kwargs)
    
    def json(self) -> dict:
        json = super().json()
        json.update({
            "stream": self.stream,
        })
        return json

class ConsoleLogger(AbstractLoggerConfig, ABC):
    def _use_console(self) -> None:
        colorised_formatter = ConfigFormatter(
            name="colorised", 
            class_name="MeowthLogger.formatters.ColorisedFormatter",
        )
        self.formatters.append(colorised_formatter)

        console_handler = ConfigConsoleHandler(
            name="console",
            class_name="logging.StreamHandler",
            level=self.settings.logger_level,
            formatter=colorised_formatter,
            stream="ext://sys.stdout",
        )
        self.handlers.append(console_handler)

        console_logger = ConfigLogger(
            name="console",
            level=self.settings.logger_level,
            propagate=False,
            handlers=[console_handler],
        )
        self.loggers.append(console_logger)


import os

from datetime import datetime, timedelta

from MeowthLogger.constants import (
    DEFAULT_LOGGING_FILENAME,
    FOLDER_DATE_FORMAT,
)
from MeowthLogger.utilities.abstractions import DateNameFile

class LogFile(DateNameFile):
    def __init__(self, 
            day: datetime,
            name: str,
            path: str
        ):
        self.path = path
        self.name = name
        hour = int(name[:2])
        self.date = day + timedelta(hours=hour)

    def read(self) -> bytes:
        with open(self.path_join, "rb") as file:
            return file.read()

class RootLogFile(LogFile):
    def __init__(self, name: str, path: str):
        self.name = name
        self.path = path

class LogDirectory(DateNameFile):
    name: str
    date: datetime

    files: list[LogFile]

    def __init__(self, name: str, path: str):
        self.name = name
        self.date = datetime.strptime(
            name,
            FOLDER_DATE_FORMAT,
        )
        self.path = path
        self.files = self.get_files()
    
    def get_files(self) -> list[LogFile]:
        files_list = os.listdir(self.path_join)

        return sorted(
            LogFile(self.date, file_name, self.path_join)
            for file_name in files_list
        )

    def read(self) -> bytes:
        return b"".join([
            file.read()
            for file in self.files
        ])

class DirectoriesTree:
    """Tree of logfiles directory
    Used for sorting files by dates
    """
    dirs: list[LogDirectory]

    def __init__(self, path) -> None:
        dir_list = os.listdir(path)

        if DEFAULT_LOGGING_FILENAME in dir_list: dir_list.remove(DEFAULT_LOGGING_FILENAME)

        self.dirs = sorted([
            LogDirectory(dir_name, path) 
            for dir_name in dir_list
        ])

    def sort_files(self, date_from, date_to) -> None:
        """Sorting self all files by requested dates
        """

        if not self.dirs:
            return

        # Filter directories list
        self.dirs = list(filter(
            lambda dir: dir.date > date_from - timedelta(days=1) and dir.date < date_to,
            self.dirs
        ))

        if self.dirs:
            # Sort files for first directory
            self.dirs[0].files = list(filter(
                lambda file: file.date > date_from - timedelta(hours=1),
                self.dirs[0].files
            ))

            # Sort files for last directory
            self.dirs[-1].files = list(filter(
                lambda file: file.date < date_to,
                self.dirs[-1].files)
            )


from ..settings import LoggerSettings

from .abstract import AbstractLoggerConfig
from .uvicorn import UvicornLogger
from .file import FileLogger
from .console import ConsoleLogger
from .stream import StreamLogger
    
class MainLoggerConfig(
        FileLogger, 
        ConsoleLogger, 
        UvicornLogger,
        StreamLogger,
        AbstractLoggerConfig,
    ):

    def __init__(
        self,
        settings: LoggerSettings,
        version: int = 1,
    ):
        super().__init__(settings, version)        
        
        self._use_console()

        if settings.use_files:
            self._use_files()

        if settings.use_uvicorn:
            self._use_uvicorn()
        
        if settings.stream:
            self._use_stream()


from typing import Optional
from logging.config import dictConfig
from logging import getLogger

from MeowthLogger.constants import (
    DEFAULT_ENCODING,
    DEFAULT_LOGGING_LEVEL,
    DEFAULT_PATH,
)
from .parser import LogParser
from .config import MainLoggerConfig
from .settings import LoggerSettings
from ..utilities.log_streaming.stream_manager import Stream

class AbstractLogger:
    def __init__(self, logger):
        self.info = logger.info
        self.critical = logger.critical
        self.debug = logger.debug
        self.error = logger.error
        self.warning = logger.warning

class Logger(LogParser, AbstractLogger):
    settings: LoggerSettings

    def __init__(
            self,
            logger_level: str = DEFAULT_LOGGING_LEVEL,
            use_files: bool = True,
            log_alive_seconds: Optional[int] = 604800,
            encoding: str = DEFAULT_ENCODING,
            path: str = DEFAULT_PATH,
            use_uvicorn: bool = False,
            stream: Optional[Stream] = None,
        ):

        self.settings = LoggerSettings(
            logger_level=logger_level,
            use_files=use_files,
            log_alive_seconds=log_alive_seconds,
            encoding=encoding,
            path=path,
            use_uvicorn=use_uvicorn,
            stream=stream
        )

        if stream:
            stream.setup_logger(self)

        dictConfig(self.__config)
        super().__init__(getLogger())

    @property
    def __config(self) -> dict:
        return MainLoggerConfig(self.settings).json()


import logging
import os
import time

from MeowthLogger.constants import DEFAULT_FORMATTER

class AbstractFormatter(logging.Formatter):
    def __init__(self):
        logging.Formatter.__init__(self, "")

    def format(self, record):
        return self.prepare_log_string(
            datetime=time.strftime(self.default_time_format, self.converter(record.created)),
            levelname=self.prepare_levelname(record.levelno),
            filename=record.pathname.replace(os.path.abspath(""), "."),
            line=str(record.lineno),
            message=record.getMessage(),
        )

    def prepare_log_string(self, datetime, levelname, filename, line, message):
        return DEFAULT_FORMATTER.format(
            datetime=datetime,
            levelname=levelname,
            filename=filename,
            line=line,
            message=message,
        )
    
    def prepare_levelname(self, levelname):
        match levelname:
            case logging.INFO:
                return "INFO"
            case logging.ERROR:
                return "ERROR"
            case logging.WARN:
                return "WARNING"
            case logging.DEBUG:
                return "DEBUG"
            case logging.CRITICAL:
                return "CRITICAL"
            case _:
                return f"LEVEL :{levelname}"


from fastapi import WebSocket, WebSocketDisconnect
from fastapi.responses import HTMLResponse

from fastapi import APIRouter
from ..log_streaming.log_streaming import get_log_stream_html


def get_log_stream_views_router(logger, router: APIRouter= None):
    if not router:
        router = APIRouter(include_in_schema=False)
    
    @router.get("/logs")
    async def get():
        html = get_log_stream_html()
        return HTMLResponse(html)

    @router.websocket("/logs")
    async def log_stream(websocket: WebSocket):
        await logger.settings.stream.connect(websocket)
        try:
            while True:
                await websocket.receive_text()
        except WebSocketDisconnect:
            logger.settings.stream.disconnect(websocket)
    
    return router
    
